@@ -2,7 +2,7 @@
  * sxid - suid, sgid file and directory checking
  * logging.c - logging functions
  *
- * Copyright (C) 1999 Ben Collins <bcollins@debian.org>
+ * Copyright (C) 1999, 2000, 2002 Ben Collins <bcollins@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -22,29 +22,310 @@
 
 #include <stdlib.h>
 #include <unistd.h>
-#include <strings.h>
 #include <stdio.h>
-#include <sys/stat.h>
 #include <sys/param.h>
 #include <limits.h>
 
-#include "structs.h"
-#include "globals.h"
-#include "notify.h"
-#include "logging.h"
-#include "search.h"
-#include "md5sum.h"
+#include "sxid.h"
 
 struct file_entry *global_log;
 FILE *mail_file;
 
+static
+char *fill_space (int count)
+{
+    char *p;
+    int i;
+
+    /* For longer lines, we try to format with some sort of tabs */
+    if (count == 0)
+	count = 5;
+    else if (count < 0) {
+	count = 0 - count;
+	for (i = 5 ; i < count ; i += 5)
+	    ;
+	count = i;
+    }
+    p = (char *) malloc (count + 1);
+    if (p == NULL) {
+	fprintf (stderr, "E: could not malloc() in fill_space()\n");
+	exit (1);
+    }
+    memset (p, ' ', count);
+    p[count] = '\0';
+    return p;
+}
+
+static
+char *format_ugid (uid_t uid, gid_t gid, mode_t mode)
+{
+    struct passwd *pw = getpwuid (uid);
+    struct group *gr = getgrgid (gid);
+    char *ret_val, *temp;
+
+    ret_val = temp = (char *) malloc ((NAME_MAX * 2) + 2);
+    if (ret_val == NULL) {
+	fprintf (stderr, "E: could not malloc() in format_ugid()\n");
+	exit (1);
+    }
+    if (mode & S_ISUID)
+	temp++[0] = '*';
+    if (pw == NULL)
+	temp += sprintf (temp, "%d", (int) uid);
+    else
+	temp += sprintf (temp, "%s", pw->pw_name);
+
+    temp++[0] = '.';
+
+    if (mode & S_ISGID)
+	temp++[0] = '*';
+    if (gr == NULL)
+	temp += sprintf (temp, "%d", (int) gid);
+    else
+	temp += sprintf (temp, "%s", gr->gr_name);
+
+    return ret_val;
+}
+
+static
+char *super_format_ugid (struct file_entry *p)
+{
+    struct passwd *pw;
+    struct group *gr;
+    char *ret_val, *temp;
+
+    ret_val = temp = (char *) malloc ((NAME_MAX * 4) + 6);
+    if (ret_val == NULL) {
+	fprintf (stderr, "E: could not malloc() in super_format_ugid()\n");
+	exit (1);
+    }
+    if (p->flags & FE_CHUSER) {
+	pw = getpwuid (p->old_uid);
+	if (pw == NULL)
+	    temp += sprintf (temp, "%d->", (int) p->old_uid);
+	else
+	    temp += sprintf (temp, "%s->", pw->pw_name);
+    }
+    pw = getpwuid (p->uid);
+    if (pw == NULL)
+	temp += sprintf (temp, "%d", (int) p->uid);
+    else
+	temp += sprintf (temp, "%s", pw->pw_name);
+
+    temp++[0] = '.';
+
+    if (p->flags & FE_CHGRP) {
+	gr = getgrgid (p->old_gid);
+	if (gr == NULL)
+	    temp += sprintf (temp, "%d->", (int) p->old_gid);
+	else
+	    temp += sprintf (temp, "%s->", gr->gr_name);
+    }
+    gr = getgrgid (p->gid);
+    if (gr == NULL)
+	temp += sprintf (temp, "%d", (int) p->gid);
+    else
+	temp += sprintf (temp, "%s", gr->gr_name);
+
+    return ret_val;
+}
+
+static
+void check_add_rem (FILE * mail_file)
+{
+    struct file_entry *p = global_log;
+    char init;
+
+    fprintf (mail_file, "Checking for any additions or removals:\n");
+
+    while (p != NULL) {
+	init = 0;
+	if (p->flags & FE_NEW)
+	    init = '+';
+	else if (p->flags & FE_OLD)
+	    init = '-';
+	if (init) {
+	    char *perms, *space1, *space2;
+
+	    perms = format_ugid (p->uid, p->gid, p->mode);
+	    space1 = fill_space (20 - strlen (perms));
+	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
+		    ? 1 : 0));
+
+	    fprintf (mail_file, " %c %s%s%s%s%s%o\n", init, p->path,
+		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
+		(int) p->mode & 0x0FFF);
+
+	    free (space1);
+	    free (space2);
+	    free (perms);
+	    changed = 1;
+	}
+	p = p->next;
+    }
+    fprintf (mail_file, "\n\n");
+    return;
+}
+
+static
+void check_attr (FILE * mail_file)
+{
+    struct file_entry *p = global_log;
+    char ino, md5;
+
+    fprintf (mail_file, "Checking for changed attributes or sums/inodes:\n");
+
+    while (p != NULL) {
+	if (p->flags & FE_CHANGEDATTR) {
+	    char old_pmode[7], *perms, *space1, *space2;
+
+	    if (p->mode & S_IFREG)
+		ino = p->flags & FE_CHSUM ? 'm' : ' ';
+	    else
+		ino = ' ';
+	    md5 = p->flags & FE_CHINODE ? 'i' : ' ';
+
+	    if (p->flags & FE_CHMODE)
+		sprintf (old_pmode, "%o->", (int) p->old_mode & 0x0FFF);
+	    else
+		old_pmode[0] = '\0';
+
+	    perms = super_format_ugid (p);
+
+	    space1 = fill_space (20 - strlen (perms));
+	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
+		    ? 1 : 0));
+
+	    fprintf (mail_file, "%c%c %s%s%s%s%s%s%o\n", ino, md5, p->path,
+		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
+		old_pmode, (int) p->mode & 0x0FFF);
+
+	    free (space1);
+	    free (space2);
+	    free (perms);
+	    changed = 1;
+	}
+	p = p->next;
+    }
+    fprintf (mail_file, "\n\n");
+    return;
+}
+
+static
+void check_u_g (FILE * mail_file)
+{
+    struct file_entry *p = global_log;
+
+    fprintf (mail_file, "Checking for no user/group matches:\n");
+
+    while (p != NULL) {
+	struct passwd *pw = getpwuid (p->uid);
+	struct group *gr = getgrgid (p->gid);
+
+	if ((pw == NULL || gr == NULL) && !(p->flags & FE_OLD)) {
+	    char *perms, *space1, *space2;
+
+	    perms = format_ugid (p->uid, p->gid, p->mode);
+	    space1 = fill_space (20 - strlen (perms));
+	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
+		    ? 1 : 0));
+
+	    fprintf (mail_file, "   %s%s%s%s%s%o\n", p->path,
+		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
+		(int) p->mode & 0x0FFF);
+
+	    free (space1);
+	    free (space2);
+	    free (perms);
+	    notify_diff = 1;
+	}
+	p = p->next;
+    }
+    fprintf (mail_file, "\n\n");
+    return;
+}
+
+static
+void check_forbidden (FILE * mail_file)
+{
+    struct file_entry *p = global_log;
+    char init;
+
+    fprintf (mail_file, "Checking for forbidden s[ug]id items:\n");
+
+    while (p != NULL) {
+	if (check_paths (p->path, config_options.forbidden_paths)) {
+	    char *perms, *space1, *space2;
+
+	    init = ' ';
+	    perms = format_ugid (p->uid, p->gid, p->mode);
+
+	    if (config_options.flags & FLAG_EFOR) {
+		p->mode &= ~(S_ISUID | S_ISGID);
+		if (chmod (p->path, p->mode))
+		    init = '!';
+		else {
+		    p->flags &= ~FE_NEW;
+		    p->flags |= FE_OLD;
+		    init = 'r';
+		}
+	    }
+	    space1 = fill_space (20 - strlen (perms));
+	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
+		    ? 1 : 0));
+
+	    fprintf (mail_file, " %c %s%s%s%s%s%o\n", init, p->path,
+		(p->mode & S_IFDIR) ? "/" : "", space2, perms,
+		space1, (int) p->mode & 0x0FFF);
+
+	    free (space1);
+	    free (space2);
+	    free (perms);
+	    notify_diff = 1;
+	}
+	p = p->next;
+    }
+
+    fprintf (mail_file, "\n\n");
+    return;
+}
+
+static
+void do_list_all (FILE * mail_file)
+{
+    struct file_entry *p = global_log;
+
+    fprintf (mail_file, "Complete list of entries:\n");
+
+    while (p != NULL) {
+	if (!p->flags & FE_OLD) {
+	    char *perms, *space1, *space2;
+
+	    perms = format_ugid (p->uid, p->gid, p->mode);
+	    space1 = fill_space (20 - strlen (perms));
+	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
+		    ? 1 : 0));
+
+	    fprintf (mail_file, "   %s%s%s%s%s%o\n", p->path,
+		(p->mode & S_IFDIR) ? "/" : "", space2, perms,
+		space1, (int) p->mode & 0x0FFF);
+
+	    free (space1);
+	    free (space2);
+	    free (perms);
+	}
+	p = p->next;
+    }
+    fprintf (mail_file, "\n\n");
+    return;
+}
+
 /* Loads the entries from the last log file */
 void init_file_entries (void)
 {
     FILE *log;
-    char buf[MAXBUF], *s, *tok, search_tok[2];
+    char buf[MAXBUF], *s, *tok, search_tok[2] = { 0x1e, '\0' };
 
-    search_tok[1] = '\0';
     global_log = NULL;
     log = fopen (config_options.log_file, "r");
     if (log == NULL)
@@ -56,13 +337,6 @@ void init_file_entries (void)
 	buf[strlen (buf) - 1] = '\0';
 	s = buf;
 
-	/* Test if we have an old format log which
-	 * is space delimited */
-	if (strchr(s, 0x1e) == NULL)
-	    search_tok[0] = ' ';
-	else
-	    search_tok[0] = 0x1e;
-
 	tok = strtok (s, search_tok);
 
 	if((config_options.flags & FLAG_SPOT) && 
@@ -247,277 +521,3 @@ void save_and_rotate (void)
     fclose (new_log);
     return;
 }
-
-char *fill_space (int count)
-{
-    char *p;
-
-    /* Always have atleast one space */
-    if (count < 1)
-	count = 1;
-    p = (char *) malloc (count + 1);
-    if (p == NULL) {
-	fprintf (stderr, "E: could not malloc() in fill_space()\n");
-	exit (1);
-    }
-    memset (p, ' ', count);
-    p[count] = '\0';
-    return p;
-}
-
-char *format_ugid (uid_t uid, gid_t gid, mode_t mode)
-{
-    struct passwd *pw = getpwuid (uid);
-    struct group *gr = getgrgid (gid);
-    char *ret_val, *temp;
-
-    ret_val = temp = (char *) malloc ((NAME_MAX * 2) + 2);
-    if (ret_val == NULL) {
-	fprintf (stderr, "E: could not malloc() in format_ugid()\n");
-	exit (1);
-    }
-    if (mode & S_ISUID)
-	temp++[0] = '*';
-    if (pw == NULL)
-	temp += sprintf (temp, "%d", (int) uid);
-    else
-	temp += sprintf (temp, "%s", pw->pw_name);
-
-    temp++[0] = '.';
-
-    if (mode & S_ISGID)
-	temp++[0] = '*';
-    if (gr == NULL)
-	temp += sprintf (temp, "%d", (int) gid);
-    else
-	temp += sprintf (temp, "%s", gr->gr_name);
-
-    return ret_val;
-}
-
-char *super_format_ugid (struct file_entry *p)
-{
-    struct passwd *pw;
-    struct group *gr;
-    char *ret_val, *temp;
-
-    ret_val = temp = (char *) malloc ((NAME_MAX * 4) + 6);
-    if (ret_val == NULL) {
-	fprintf (stderr, "E: could not malloc() in super_format_ugid()\n");
-	exit (1);
-    }
-    if (p->flags & FE_CHUSER) {
-	pw = getpwuid (p->old_uid);
-	if (pw == NULL)
-	    temp += sprintf (temp, "%d->", (int) p->old_uid);
-	else
-	    temp += sprintf (temp, "%s->", pw->pw_name);
-    }
-    pw = getpwuid (p->uid);
-    if (pw == NULL)
-	temp += sprintf (temp, "%d", (int) p->uid);
-    else
-	temp += sprintf (temp, "%s", pw->pw_name);
-
-    temp++[0] = '.';
-
-    if (p->flags & FE_CHGRP) {
-	gr = getgrgid (p->old_gid);
-	if (gr == NULL)
-	    temp += sprintf (temp, "%d->", (int) p->old_gid);
-	else
-	    temp += sprintf (temp, "%s->", gr->gr_name);
-    }
-    gr = getgrgid (p->gid);
-    if (gr == NULL)
-	temp += sprintf (temp, "%d", (int) p->gid);
-    else
-	temp += sprintf (temp, "%s", gr->gr_name);
-
-    return ret_val;
-}
-
-void check_add_rem (FILE * mail_file)
-{
-    struct file_entry *p = global_log;
-    char init;
-
-    fprintf (mail_file, "Checking for any additions or removals:\n");
-
-    while (p != NULL) {
-	init = 0;
-	if (p->flags & FE_NEW)
-	    init = '+';
-	else if (p->flags & FE_OLD)
-	    init = '-';
-	if (init) {
-	    char *perms, *space1, *space2;
-
-	    perms = format_ugid (p->uid, p->gid, p->mode);
-	    space1 = fill_space (20 - strlen (perms));
-	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
-		    ? 1 : 0));
-
-	    fprintf (mail_file, " %c %s%s%s%s%s%o\n", init, p->path,
-		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
-		(int) p->mode & 0x0FFF);
-
-	    free (space1);
-	    free (space2);
-	    free (perms);
-	    changed = 1;
-	}
-	p = p->next;
-    }
-    fprintf (mail_file, "\n\n");
-    return;
-}
-
-void check_attr (FILE * mail_file)
-{
-    struct file_entry *p = global_log;
-    char ino, md5;
-
-    fprintf (mail_file, "Checking for changed attributes or sums/inodes:\n");
-
-    while (p != NULL) {
-	if (p->flags & FE_CHANGEDATTR) {
-	    char old_pmode[7], *perms, *space1, *space2;
-
-	    if (p->mode & S_IFREG)
-		ino = p->flags & FE_CHSUM ? 'm' : ' ';
-	    else
-		ino = ' ';
-	    md5 = p->flags & FE_CHINODE ? 'i' : ' ';
-
-	    if (p->flags & FE_CHMODE)
-		sprintf (old_pmode, "%o->", (int) p->old_mode & 0x0FFF);
-	    else
-		old_pmode[0] = '\0';
-
-	    perms = super_format_ugid (p);
-
-	    space1 = fill_space (20 - strlen (perms));
-	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
-		    ? 1 : 0));
-
-	    fprintf (mail_file, "%c%c %s%s%s%s%s%s%o\n", ino, md5, p->path,
-		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
-		old_pmode, (int) p->mode & 0x0FFF);
-
-	    free (space1);
-	    free (space2);
-	    free (perms);
-	    changed = 1;
-	}
-	p = p->next;
-    }
-    fprintf (mail_file, "\n\n");
-    return;
-}
-
-void check_u_g (FILE * mail_file)
-{
-    struct file_entry *p = global_log;
-
-    fprintf (mail_file, "Checking for no user/group matches:\n");
-
-    while (p != NULL) {
-	struct passwd *pw = getpwuid (p->uid);
-	struct group *gr = getgrgid (p->gid);
-
-	if ((pw == NULL || gr == NULL) && !(p->flags & FE_OLD)) {
-	    char *perms, *space1, *space2;
-
-	    perms = format_ugid (p->uid, p->gid, p->mode);
-	    space1 = fill_space (20 - strlen (perms));
-	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
-		    ? 1 : 0));
-
-	    fprintf (mail_file, "   %s%s%s%s%s%o\n", p->path,
-		(p->mode & S_IFDIR) ? "/" : "", space2, perms, space1,
-		(int) p->mode & 0x0FFF);
-
-	    free (space1);
-	    free (space2);
-	    free (perms);
-	    notify_diff = 1;
-	}
-	p = p->next;
-    }
-    fprintf (mail_file, "\n\n");
-    return;
-}
-
-void check_forbidden (FILE * mail_file)
-{
-    struct file_entry *p = global_log;
-    char init;
-
-    fprintf (mail_file, "Checking for forbidden s[ug]id items:\n");
-
-    while (p != NULL) {
-	if (check_paths (p->path, config_options.forbidden_paths)) {
-	    char *perms, *space1, *space2;
-
-	    init = ' ';
-	    perms = format_ugid (p->uid, p->gid, p->mode);
-
-	    if (config_options.flags & FLAG_EFOR) {
-		p->mode &= ~(S_ISUID | S_ISGID);
-		if (chmod (p->path, p->mode))
-		    init = '!';
-		else {
-		    p->flags &= ~FE_NEW;
-		    p->flags |= FE_OLD;
-		    init = 'r';
-		}
-	    }
-	    space1 = fill_space (20 - strlen (perms));
-	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
-		    ? 1 : 0));
-
-	    fprintf (mail_file, " %c %s%s%s%s%s%o\n", init, p->path,
-		(p->mode & S_IFDIR) ? "/" : "", space2, perms,
-		space1, (int) p->mode & 0x0FFF);
-
-	    free (space1);
-	    free (space2);
-	    free (perms);
-	    notify_diff = 1;
-	}
-	p = p->next;
-    }
-
-    fprintf (mail_file, "\n\n");
-    return;
-}
-
-void do_list_all (FILE * mail_file)
-{
-    struct file_entry *p = global_log;
-
-    fprintf (mail_file, "Complete list of entries:\n");
-
-    while (p != NULL) {
-	if (!p->flags & FE_OLD) {
-	    char *perms, *space1, *space2;
-
-	    perms = format_ugid (p->uid, p->gid, p->mode);
-	    space1 = fill_space (20 - strlen (perms));
-	    space2 = fill_space (32 - strlen (p->path) - ((p->mode & S_IFDIR)
-		    ? 1 : 0));
-
-	    fprintf (mail_file, "   %s%s%s%s%s%o\n", p->path,
-		(p->mode & S_IFDIR) ? "/" : "", space2, perms,
-		space1, (int) p->mode & 0x0FFF);
-
-	    free (space1);
-	    free (space2);
-	    free (perms);
-	}
-	p = p->next;
-    }
-    fprintf (mail_file, "\n\n");
-    return;
-}
